Package: clusterSeq
Type: Package
Title: Clustering of high-throughput sequencing data by identifying
        co-expression patterns
Version: 1.36.0
Depends: R (>= 3.0.0), methods, BiocParallel, baySeq, graphics, stats,
        utils
Imports: BiocGenerics
Suggests: BiocStyle
Date: 2016-01-19
Authors@R: c(person("Thomas J.", "Hardcastle", role = "aut"),
  person("Irene", "Papatheodorou", role = "aut"),
  person("Samuel", "Granjeaud", role = "cre", email = "samuel.granjeaud@inserm.fr", comment=c(ORCID="0000-0001-9245-1535")))
Description: Identification of clusters of co-expressed genes based on
        their expression across multiple (replicated) biological
        samples.
License: GPL-3
LazyLoad: yes
biocViews: Sequencing, DifferentialExpression, MultipleComparison,
        Clustering, GeneExpression
URL: https://github.com/samgg/clusterSeq
BugReports: https://github.com/samgg/clusterSeq/issues
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-04-28 12:44:49 UTC
RemoteUrl: https://github.com/bioc/clusterSeq
RemoteRef: RELEASE_3_23
RemoteSha: e53b3fc46a6e2a40526b535daa3e7e6b60c388ca
NeedsCompilation: no
Packaged: 2026-04-29 20:40:22 UTC; root
Author: Thomas J. Hardcastle [aut],
  Irene Papatheodorou [aut],
  Samuel Granjeaud [cre] (ORCID: <https://orcid.org/0000-0001-9245-1535>)
Maintainer: Samuel Granjeaud <samuel.granjeaud@inserm.fr>
Built: R 4.6.0; ; 2026-04-29 20:48:29 UTC; unix

importFrom("graphics", "lines", "plot")
importFrom("stats", "as.dist", "cutree", "hclust", "kmeans", "median", "runif", "sd")
importFrom("utils", "object.size", "read.table")
importFrom("baySeq", "libsizes")
importFrom("BiocGenerics", "unlist")
import("methods", "BiocParallel")
export("kCluster", "associatePosteriors", "plotCluster", "makeClusters", "makeClustersFF", "wallace")

# modification on git from copied files
kCluster <-

    function(cD, maxK = 100, matrixFile = NULL, replicates = NULL, algorithm = "Lloyd", B = 1000, sdm = 1)
{

    kstats <- function(k, x, cx) {
        if(k > length(unique(x))) return(rep(NA, 3))
        if(k == length(x)) {
                                        #if(is.null(replicates) || length(replicates) == length(unique(replicates))) {
            clustK <- list(centers = x, cluster = seq_along(x))
                                        #} else return(c(NA, NA))
        } else {
            clustK <- suppressWarnings(kmeans(x, k, iter.max = 1000, nstart = 100, algorithm = algorithm))
            if(any(is.na(clustK$centers))) clustK <- suppressWarnings(kmeans(x, k, iter.max = 1000, nstart = 100, algorithm = algorithm))
                                        #store the clustering and orderings as a string
            
                                        # the match bit makes sure that identical clusterings have identical numberings
                                        #if(forceReplicates && any(sapply(lapply(split(clustK$cluster, replicates), unique), length) > 1)) return(rep(NA, 2))                
        }
        if(forceReplicates) cluster <- clustK$cluster[match(replicates, levels(replicates))] else cluster <- clustK$cluster
        
        clusterings <- paste(match(cluster, (unique(cluster))), collapse = ":")
        orderings <- paste(order(clustK$centers[unique(cluster)]), collapse = "<")
        clusterOrder <- paste(clusterings, orderings, sep = "-")
                                        # statistic gets stored as a string too, with marginal loss of precision. Could fix it, but who cares
                                        #            stat <- max(sapply(seq_len(k), function(kk) {
                                        #                mx <- suppressWarnings(max(abs(split(cx, factor(cluster, levels = seq_len(k)))[[kk]] - clustK$centers[kk]))#
                                        #                mx
                                        #            }))
        
        stat = max(sapply(split(cx, factor(cluster, levels = seq_len(k))), function(kx) diff(range(kx))))
                                        #wss <- sum(sapply(split(cx, factor(cluster, levels = seq_len(k))), function(kx) sum((kx - mean(kx))^2)))
        
                                        #if(k > 1) dstat <- stat / min(diff(sort(clustK$centers))) else dstat <- NA
                                        #W <- sum(clustK$withinss)
        dstat <- stat / diff(range(cx))
        c(clusterOrder, stat, dstat)
    }
    
    
    if(inherits(cD, "countData")) {
        dat <- cD@data
        dat[dat == 0] <- 1
        dat <- log2(t(t(dat) / as.vector(libsizes(cD)) * mean(libsizes(cD))))
        replicates <- cD@replicates        
    } else dat <- as.matrix(cD)
    if(is.null(replicates)) {
        forceReplicates <- FALSE
    } else {
        if(!is.factor(replicates)) replicates <- as.factor(replicates)
        forceReplicates <- TRUE
        mdat <- do.call("cbind", lapply(levels(replicates), function(rep) apply(dat[,replicates == rep], 1, median)))
        if(length(replicates) != ncol(cD)) stop("If replicates are provided, there must be the same number of elements in the 'replicates' vector as there are columns in the 'cD' object.")
    }
    #print(forceReplicates)

    # this looks like it's something like Tibshirani's gap statistic - simulate data on a uniform distribution
    mx <- matrix(runif(n = B * ncol(dat), min = 0, max = 1), ncol = ncol(dat))
    if(forceReplicates) mxx <- t(apply(mx, 1, function(mxx) sapply(split(mxx, replicates), median))) else mxx <- mx
    
    mpK <- min(ncol(mxx), maxK)

    # calculate kstats based on that uniform distribution
    message("Bootstrapping distributions...", appendLF = TRUE) 
    pseudoW <- do.call("rbind", bplapply(seq_len(nrow(mx)), function(kk) 
        do.call("cbind", lapply(seq_len(mpK), kstats, x = mxx[kk,], cx = mx[kk,]))[2,]))
                                       #, BPPARAM = BP(workers = as.integer(cores), progressbar = TRUE)))
    message("done!")
    
    message("K-means processing...", appendLF = FALSE)

    koverk <- function(ii, maxK, forceReplicates, replicates) {
        if(forceReplicates) x <- mdat[ii,] else x <- dat[ii,]
        cx <- dat[ii,]
        
        genestats <- sapply(seq_len(min(length(x), maxK)), kstats, x = x, cx = cx)
        
        bootExp <- function(x, cx, replicates, forceReplicates) {
            rx <- range(cx)
            if(diff(rx) == 0) return(1)
            # uniform distribution gets rescaled here to range of data
            lWs <- split(log(as.numeric(pseudoW) * diff(rx)), rep(seq_len(mpK), each = B))
            elWk <- sapply(lWs, mean)
            lW <- log(as.numeric(genestats[2,seq_len(mpK)]))
            gap <- elWk - lW
            sdk <- sqrt(1 + 1/B) * sapply(lWs, sd)
            suppressWarnings(clustNum <- min(which(gap[-length(gap)] > gap[-1] - sdm * sdk[-1])))
                                        #mono <- gap[-length(gap)] > gap[-1] - sdm * sdk[-1]
            clustNum
        }

        clustNum <- bootExp(x, cx, replicates = replicates, forceReplicates = forceReplicates)
        clustNumM <- bootExp(x, x, replicates = NULL, forceReplicates = FALSE)
        
        list(gs = genestats, clustNum, clustNumM)
    }
                                       # this bit does all possible clusterings, parallelised
    kgene <- do.call("cbind", bplapply(seq_len(nrow(dat)), koverk, maxK = maxK, forceReplicates = forceReplicates, replicates = replicates))#, BPPARAM = BP(workers = as.integer(cores), progressbar = TRUE)))

    message(".done!")

    monos <- unlist(kgene[2,]) == 1
    
                                        # fix dimensions of matrix 
    kgeneS <- do.call("cbind", kgene[1,])
    
 
                                        # separate out clusterings and stats
    clusterings <- matrix(kgeneS[1,], ncol = nrow(dat))
    clusterings[is.na(clusterings)] <- FALSE
    stats <- matrix(as.numeric(kgeneS[3,]), ncol = nrow(dat))

    stats <- round(stats, 3)

                                        #temporary matrix prototype
    infmat <- matrix(1, nrow = nrow(stats), ncol = ncol(stats))

    stats[1,monos] <- 0
#    stats[seq.int(2,nrow(stats)),monos] <- Inf

    if(!is.null(matrixFile)) {
        if(substr(matrixFile, nchar(matrixFile) -2, nchar(matrixFile)) != ".gz") {
            message("Matrix file will be gzipped; appending '.gz' to filename supplied")
            matrixFile <- paste(matrixFile, ".gz", sep = "")
        }
        if(file.exists(matrixFile)) file.remove(matrixFile)
        gzfile <- gzfile(matrixFile, "w")
    }



                                        # write one by one the stats for each gene. Can't trivially parallelise this bit or you will have multiple threads writing to same file. Could parallelise with writes to separate files followed by cat and sort but this would take it out of pure R.


    message("Comparing clusterings...", appendLF = FALSE)

    if(!is.null(matrixFile)) {
        lapplyFun <- lapply
        writeLines(paste(c("", rownames(dat)), collapse = "\t"), gzfile)
    } else lapplyFun <- bplapply
    kAM <- do.call("rbind", lapplyFun(seq_len(ncol(clusterings)), function(ii) {
            if(sample(seq_len(100), 1) == 1) message(".", appendLF = FALSE)
                                          # fill in temporary matrix with valid statistics for each comparison
            tmat <- infmat; tmat[(clusterings[,ii] == clusterings)] <- stats[clusterings[,ii] == clusterings]
                                        # update temporary matrix with stats from current gene (if bigger)
            tmat <- apply(cbind(stats[,ii], tmat), 1, function(x) pmax(x[-1], x[1]))
            
                                        # select minimum statistic from each column of temporary matrix and write

            minstats <- do.call(pmin, c(lapply(seq_len(ncol(tmat)), function(i)tmat[,i]), list(na.rm = TRUE)))
            if(!is.null(matrixFile))
                writeLines(paste(rownames(dat)[ii], paste(minstats, collapse = "\t"), sep = "\t"), gzfile)
            minstats[seq_len(ii)] <- 1
            c(id = ii, pair.id = which.min(minstats), stat = min(minstats))
        }))#, BPPARAM = BP(workers = as.integer(cores), progressbar = TRUE)))

    if(!is.null(matrixFile)) close(gzfile)
    message(".done!")
    kAM
}

# modification on git from copied files
associatePosteriors <-
function(cD, maxsize = 250000, matrixFile = NULL) {    
    matord <- do.call("rbind", lapply(seq_len(ncol(cD@orderings)), function(ii) as.integer(cD@orderings[,ii])))
    posteriors <- t(cD@posteriors)
    
    objsize <- as.numeric(gsub(" Mb", "", format(object.size(posteriors), units = "Mb"))) + as.numeric(gsub(" Mb", "", format(object.size(matord), units = "Mb")))    
    splits <- ceiling(objsize / sqrt(maxsize))
    
                                        #clusterExport(cl, c("matord", "posteriors"), envir = environment())
    if(splits == 1) splosts <- list(seq_len(ncol(posteriors))) else splosts <- split(seq_len(ncol(posteriors)), cut(seq_len(ncol(posteriors)),
           splits, labels = FALSE))
    
    
    aMM <- NULL
    for(ss in seq_along(splosts)) {
        splsub <- splosts[[ss]]
        
        if(!is.null(matrixFile)) {
            if(substr(matrixFile, nchar(matrixFile) -2, nchar(matrixFile)) != ".gz") {
                message("Matrix file will be gzipped; appending '.gz' to filename supplied")
                matrixFile <- paste(matrixFile, ".gz", sep = "")
            }
            if(file.exists(matrixFile)) file.remove(matrixFile)
            gzfile <- gzfile(matrixFile, "w")
        }
        
        saM <- do.call("rbind", lapply(ss:length(splosts), function(jj) {
            message(".", appendLF = FALSE)
            rposts <- posteriors[,splosts[[jj]]];
            rmatord <- matord[,splosts[[jj]]];

            if(!is.null(matrixFile)) lapplyFun <- lapply else lapplyFun <- bplapply
            aM <- cbind(splsub, do.call("rbind", lapplyFun(splsub,
                                                          function(ii) {
                                                              posts <- colSums(exp(rposts + log(rmatord == matord[,ii]) + posteriors[,ii]))
                                                              
                                                              posts[splosts[[jj]] <= ii] <- NA
                                                              if(!is.null(matrixFile))
                                                                  writeLines(paste(posts, collapse = "\t"), gzfile)

                                                              c(splosts[[jj]][which.max(posts)], max(posts, na.rm = TRUE))
                                                              
                                                          })))
            aM[aM[,3] > -Inf,]
        }))

        message("", appendLF = TRUE)

        if(!is.null(matrixFile)) close(gzfile)
        
        aMM <- rbind(aMM, saM)
    }
    
    aMM
    splam <- split(as.data.frame(aMM, stringsAsFactors = FALSE), aMM[,1])
    sAMM <- do.call("rbind", lapply(splam, function(x) x[which.max(x[,3]),]))
    colnames(sAMM) <- c("id", "pair.id", "stat")
    sAMM$stat <- 1 - sAMM$stat
    sAMM
}

# modification on git from copied files
.makeClusters <-
function(assocMat, threshold = 0.5) {
    clM <- assocMat
    diag(clM) <- NA
    clM[clM > threshold] <- NA
    rownames(clM) <- colnames(clM) <- seq_len(nrow(clM))
    mines <- apply(clM, 1, function(x) if(all(is.na(x))) NA else which.min(x))
    clcon <- cbind(seq_len(nrow(clM)), mines)
    clcon <- clcon[!is.na(clcon[,1]) & !is.na(clcon[,2]),]
    clcon <- t(apply(clcon, 1, sort))
    clcon <- clcon[!duplicated(clcon),]

    clusterList <- list()
    while(TRUE) {
        initcon <- clcon[1,1]
        while(TRUE) {
            newcon <- unique(as.vector(clcon[clcon[,1] %in% initcon | clcon[,2] %in% initcon,]))
            if(all(newcon %in% initcon)) break()
            initcon <- newcon
        }
        clcon <- clcon[!(clcon[,1] %in% initcon | clcon[,2] %in% initcon),,drop=FALSE]
        clusterList <- c(clusterList, list(sort(as.numeric(initcon))))
        if(length(clcon) == 0) break()
    }
    clusterList <- clusterList[order(sapply(clusterList, length), decreasing = TRUE)]
    clusterList
}

# modification on git from copied files
plotCluster <-
function(cluster, cD, sampleSize = 1000) {
    if(inherits(cD, "countData")) {
        dat <- cD@data
        dat[dat == 0] <- 1
        dat <- log2(t(t(dat) / as.vector(libsizes(cD)) * mean(libsizes(cD))))
    } else dat <- cD

    .plotClust <- function(cluster, cD, main) {
        sampclust <- (sample(seq_along(cluster), size = min(sampleSize, length(cluster)), replace = FALSE))
        rdat <- dat[cluster[sampclust],]
        mrdat <- apply(rdat, 1, function(x) {
            y <- x - min(x)
            y / max(c(1, x))
        })
        
        plot(x = NA, y = NA, xlim = c(1,ncol(dat)), ylim = c(min(mrdat), max(mrdat)), main = main, ylab = "row-normalised scale", xlab = "sample")
        apply(mrdat, 2, function(x) lines(x))
    }

    if(inherits(cluster, "SimpleIntegerList")) for(ii in seq_along(cluster)) .plotClust(cluster[[ii]], cD, main = names(cluster)[ii]) else .plotClust(cluster, cD, "")
    return(NULL)
}

# modification on git from copied files
.nameList <- function(x, cD) {
    groups <- which(colSums(cD@posteriors[x,,drop = FALSE] > log(0.5)) > length(x) * 0.1)
    if(length(groups) == 0) return("")
    groupD <- sapply(groups, function(ii) {
        grp <- cD@groups[[ii]]
        groupDesc <- lapply(levels(grp), function(gg) {
            grprep <- unique(cD@replicates[grp == gg])
            paste(grprep, collapse = "=")
        })
        groupDesc <- lapply(groupDesc, function(x) paste("{", x, "}", sep = ""))
        if(length(groupDesc) == 1) return(groupDesc[[1]])
        orders <- cD@orderings[x,ii][cD@posteriors[x,ii] > log(0.5)]
        lapply(levels(orders)[table(orders) > 0.1 * length(orders)], function(ordering) {                
            paste(as.vector(rbind(unlist(strsplit(as.character(ordering), "[^(>|=)]")), unlist(groupDesc[as.numeric(strsplit(as.character(ordering), ">|=")[[1]])]))), collapse = "")
        })    
    }
                     )
    paste(unlist(groupD), collapse = ";")
}


makeClusters <-
function(aM, cD, threshold = 0.5) {
    greaterThan = FALSE
    if(greaterThan) aM <- aM[aM[,3] > threshold,] else aM <- aM[aM[,3] < threshold,]
    
    x <- aM[,seq_len(2)]
    for(ii in nrow(x):1) {
        rx <- which(x[,1] == x[ii,2])
        if(length(rx) > 0)
            x[ii,2] <- x[rx,2]
    }
    colnames(x) <- c("cl1", "cl2")
    rx <- rbind(x, cbind("cl1" = unique(x[,2]), "cl2" = unique(x[,2])))
    sx <- split(rx[,1], rx[,2])
    sx <- sx[order(sapply(sx, length), decreasing = TRUE)]

    sx <- c(sx, as.list(which(!seq_len(nrow(cD)) %in% unlist(sx))))
    
    if(inherits(cD, "countData")) names(sx) <- sapply(sx, .nameList, cD = cD)
    sx <- as(sx, "IntegerList")
    sx
}


makeClustersFF <- function(file, method = "complete", cut.height = 5)
{
    kmat <- read.table(file, header = TRUE, row.names=1)
    kmat[kmat == Inf] <- 1e6
    dkmat <- as.dist(kmat)

    fit <- hclust(dkmat, method = method)
    fit$height <- round(fit$height, 6)
    
    groups <- cutree(fit, h = cut.height)
    mqGroups <- split(seq_len(nrow(kmat)), groups)
                                        #names(mqGroups) <- sapply(mqGroups, nameList, cD = cD)
    mqGroups <- mqGroups[order(sapply(mqGroups, length), decreasing = TRUE)]
    mqGroups <- as(mqGroups, "IntegerList")
    mqGroups
}

# modification on git from copied files
.clust2Vect <- function(x) {
    vec <- rep(NA, length(x))
    vec[unlist(x)] <- rep(seq_along(x), sapply(x, length))   
    if(any(is.na(vec))) vec[is.na(vec)] <- seq_len(sum(is.na(vec))) + max(vec,na.rm = TRUE)
    vec
}



wallace <- function(v1, v2) {
    if(inherits(v1, "SimpleIntegerList")) v1 <- .clust2Vect(v1)
    if(inherits(v2, "SimpleIntegerList")) v2 <- .clust2Vect(v2)
    confusion <- table(v1, v2)
    confusion <- confusion[rowSums(confusion) > 1, colSums(confusion) > 1]
    N11 <- sum(confusion * (confusion - 1) / 2)
    n1 <- table(v1); n2 <- table(v2)
    c(P.v1_given_v2 = N11 / sum(n1 * (n1 - 1) / 2), 
      P.v2_given_v1 = N11 / sum(n2 * (n2 - 1) / 2))    
}

.fowlkesMallows <- function(v1, v2) {
    sqrt(prod(wallace(v1, v2)))
}

.mirkin <- function(v1, v2) {
    n1 <- table(v1); n2 <- table(v2)
    confusion <- table(v1, v2)
    sum(n1^2) + sum(n2^2) - 2 * sum(confusion^2)
}

% git mod
%\VignetteIndexEntry{Advanced baySeq analyses}
%\VignettePackage{baySeq}
%\VignetteKeywords{baySeq, generic, advanced}

\documentclass[a4paper]{article}

\title{Clustering high-throughput sequencing data based on patterns of co-expression}
\author{Thomas J. Hardcastle}

<<<style-Sweave, eval=TRUE, echo=FALSE, results=tex>>=
BiocStyle::latex()
@ 

\begin{document}

\maketitle

\section{Introduction}

This vignette outlines two possibilities for clustering gene expression based on patterns of co-expression between samples, optionally accounting for the replicate structure of the data.

The package can be installed as
<<eval=FALSE>>=
if (!requireNamespace("BiocManager", quietly=TRUE))
    install.packages("BiocManager")
BiocManager::install("clusterSeq")
@

<<>>=
library(clusterSeq)
@ 

% data are filtered for minimum expression
%#

We demonstrate these analyses on a set of time series data in female rat thymus tissues taken from the Rat Bodymap project \cite{Yu2014}. By identifying clusters of genes that demonstrate similar patterns of expression over time we can identify patterns of time dependence within the data.

We first load in the processed data of observed read counts at each gene for each sample. 
<<results = hide>>=
data(ratThymus, package = "clusterSeq")
head(ratThymus)
@

We define the replicate structure of the data in a vector whose members correspond to the columns of the data matrix.
<<>>=
replicates <- c("2week","2week","2week","2week",
                "6week","6week","6week","6week",
                "21week","21week","21week","21week",
                "104week","104week","104week","104week")
@

Library scaling factors are acquired here using the \verb'baySeq::getLibsizes' function but might be acquired through any other means.

<<>>=
  library(baySeq)
  libsizes <- getLibsizes(data = ratThymus)
@

\section{K-means based clustering}

For k-means based clustering, we assume an approximately log-normal distribution. We adjust the data to remove zeros and rescale by the library scaling factors.
<<>>=
ratThymus[ratThymus == 0] <- 1
normRT <- log2(ratThymus %*% diag(1/libsizes) * mean(libsizes))
@ 

For speed purposes, we  will consider only the first 1000 genes in the data.
<<>>=
normRT <- normRT[1:1000,]
@ 

K-means based clustering compares two genes by separately clustering the expression values of each gene using all possible values of $k$ (or, if the sample size is exceptionally large, all values of $k$ from 1 to some large maximum permitted value.). The genes are then compared to find the maximum $k$ for which the average expression of the identified clusters is monotonic between the two genes. For this value of $k$, the the maximum difference between expression levels observed within a cluster of either gene is reported as a measure of the dissimilarity between the two genes.

However, a problem arises for genes that are non-differentially expressed across all samples. For these genes, the appropriate number of clusters is 1; however, choices of $k > 1$ often lead to false assignment of genes to clusters exhibiting differential expression. To resolve this, the clustering incorporates a bootstrapping stage based on Tibshirani's gap statistic. Bootstrapping uniformly distributed data on the same range as the observed data, we calculate the the dissimilarity score as above, and find those cases for which the gap between the bootstrapped mean dissimilarity and the observed dissimilarity for k = 1 exceeds that for k = 2 by more than some multiple of the standard error of the bootstrapped dissimilarities of k = 2. These cases are forced to be treated as non-differentially expressed by discarding all dissimilarity data for k > 1.

We generate a matrix of dissimilarity scores between each pair of genes using the \verb'kCluster' function. The full matrix can be reported to (gzipped) file; the function returns a data.frame which for each gene defines its nearest neighbour of higher row index, and the dissimilarity with that neighbour. This is sufficient for a clustering based on singleton agglomeration (see \verb'hclust').

<<results = hide>>=
kClust <- kCluster(normRT, matrixFile = "kclust_matrix.txt.gz", B = 1000)
head(kClust)
@

Using the output from 'kCluster' we construct a clustering of genes by singleton agglomeration of those genes with dissimilarity scores lower than a specified threshold.
<<>>=
mkClust <- makeClusters(kClust, normRT, threshold = 1)
@ 

We can repeat this analysis forcing members of the same replicate groups to cluster together at the k-means stage.

<<results = hide>>=
kClustR <- kCluster(normRT, replicates = replicates, matrixFile = "kclust_matrix_newForceReps.txt.gz", B = 1000)
mkClustR <- makeClusters(kClustR, normRT, threshold = 1)
@

As an alternative to singleton agglomeration, we can use the full matrix and any agglomerative method defined in the \verb'hclust' heirarchical analysis function.
<<>>=
mkClustRC <- makeClustersFF("kclust_matrix_newForceReps.txt.gz", method = "complete", cut.height = 5)
@ 


%<<>>=
%summariseStability(mkClustR, mkClust)
%@ 

\section{Posterior likelihood analyses}
         
As an alternative to the K-means based clustering, we can use empirical Bayesian likelihoods on all possible models for the structure of the data at each gene, and from these estimate the likelihood that two genes share the same structure. We estimate these likelihoods using the \verb'baySeq' package. 

We first create a \verb'countData' object to contain the data and find all possible models that preserve the replicate structure.

<<>>=
library(baySeq)
cD.ratThymus <- new("countData", data = ratThymus, replicates = c(1,1,1,1,2,2,2,2,3,3,3,3,4,4,4,4))
densityFunction(cD.ratThymus) <- nbinomDensity
libsizes(cD.ratThymus) <- getLibsizes(cD.ratThymus)
cD.ratThymus <- allModels(cD.ratThymus)
@ 

The commented out code is for the analysis of the data with baySeq. Since this takes some time, this has already been performed and can be loaded in with the analysis already performed.
<<>>=
#cl <- makeCluster(4)
#cD.ratThymus <- getPriors(cD.ratThymus, consensus = TRUE, cl = cl)
#cD.ratThymus <- getLikelihoods(cD.ratThymus, cl = cl)

data(cD.ratThymus, package = "clusterSeq")
@ 

Given the estimated posteriors for each gene to have each model, we estimate the likelihood that two genes have identical models, and that the ordering of average expression in the groups defined by those models is monotonic between the two genes. As before, the function \verb'associatePosteriors' returns a data.frame which for each gene defines its nearest neighbour of higher row index, and the dissimilarity with that neighbour. This is sufficient for a clustering based on singleton agglomeration using the \verb'makeClusters' function.

<<>>=
cD.ratThymus <- cD.ratThymus[1:1000,]
aM <- associatePosteriors(cD.ratThymus)
sX <- makeClusters(aM, cD.ratThymus, threshold = 0.5)
@ 

Likelihoods of a pair being clustered together in one clustering given that they are clustered in that way in the other.
<<>>=
wallace(sX, mkClustRC)
@ 

<<results = hide, fig = FALSE, label=plotClusterBS>>=
par(mfrow = c(2,3))
plotCluster(sX[1:6], cD.ratThymus)
@

\begin{figure}[!ht]
\begin{center}
 <<label=plotClusterBS-fig, fig=TRUE,echo=FALSE>>=
 <<plotClusterBS>>
 @
\caption{Profiles of gene expression in top six clusters acquired from baySeq analysis.}
\label{figPCBS}
\end{center}
\end{figure}


\section*{Session Info}

<<>>=
sessionInfo()
@ 

\begin{thebibliography}{99}
\bibitem{Yu2014} Yu, Ying and Fuscoe, James C and Zhao, Chen and Guo, Chao and Jia, Meiwen and Qing, Tao and Bannon, Desmond I and Lancashire, Lee and Bao, Wenjun and Du, Tingting and Luo, Heng and Su, Zhenqiang and Jones, Wendell D and Moland, Carrie L and Branham, William S and Qian, Feng and Ning, Baitang and Li, Yan and Hong, Huixiao and Guo, Lei and Mei, Nan and Shi, Tieliu and Wang, Kevin Y and Wolfinger, Russell D and Nikolsky, Yuri and Walker, Stephen J and Duerksen-Hughes, Penelope and Mason, Christopher E and Tong, Weida and Thierry-Mieg, Jean and Thierry-Mieg, Danielle and Shi, Leming and Wang, Charles \textsl{A rat RNA-Seq transcriptomic BodyMap across 11 organs and 4 developmental stages.} Nature Communications (2014).
\end{thebibliography}


\end{document}

% modification on git from copied files
\name{makeClusters}
\alias{makeClusters}

\title{Creates clusters from a co-expression minimal linkage data.frame.}

\description{
This function uses minimal linkage data to perform rapid clustering by
singleton agglomeration (i.e., a gene will always cluster with its
nearest neighbours provided the distance to those neighbours does not
exceed some threshold). For alternative (but slower) clustering options,
see the \code{\link{makeClustersFF}} function.
}
\usage{
makeClusters(aM, cD, threshold = 0.5)
}

\arguments{
  \item{aM}{
    A data frame constructed by \code{\link{associatePosteriors}} or
    \code{\link{kCluster}}, defining for each gene the nearest
    neighbour of higher row index and the dissimilarity with that neighbour.
}
  \item{cD}{
    The data given as input to \code{\link{associatePosteriors}} or
    \code{\link{kCluster}} that produced 'aM'.
}
  \item{threshold}{
    A threshold on the maximum dissimilarity at which two genes can
    cluster. Defaults to 0.5.
}
}

\value{
An IntegerList object, each member of whom defines a cluster of co-expressed
genes. The object is ordered decreasingly by the size of each cluster.
}

\author{
Thomas J Hardcastle
}
\seealso{
  \code{\link{makeClustersFF}}
  \code{\link{kCluster}}
  \code{\link{associatePosteriors}}
}
\examples{
#Load in the processed data of observed read counts at each gene for each sample. 
data(ratThymus, package = "clusterSeq")

# Library scaling factors are acquired here using the getLibsizes
# function from the baySeq package.
libsizes <- getLibsizes(data = ratThymus)

# Adjust the data to remove zeros and rescale by the library scaling
# factors. Convert to log scale.
ratThymus[ratThymus == 0] <- 1
normRT <- log2(t(t(ratThymus / libsizes)) * mean(libsizes))

# run kCluster on reduced set.
normRT <- normRT[1:1000,]
kClust <- kCluster(normRT)

# make the clusters from these data.
mkClust <- makeClusters(kClust, normRT, threshold = 1)

}

\keyword{manip}

% modification on git from copied files
\name{clusterSeq-package}
\alias{clusterSeq-package}
\alias{clusterSeq}
\docType{package}
\title{
\packageTitle{clusterSeq}
}
\description{
Identification of clusters of co-expressed genes based on their
expression across multiple (replicated) biological samples.}

\details{
The DESCRIPTION file:
\packageDESCRIPTION{clusterSeq}
\packageIndices{clusterSeq}
}
\author{
\packageAuthor{clusterSeq}

Maintainer: \packageMaintainer{clusterSeq}
}

\keyword{package}

\examples{
#Load in the processed data of observed read counts at each gene for each sample. 
data(ratThymus, package = "clusterSeq")

# Library scaling factors are acquired here using the getLibsizes
# function from the baySeq package.
libsizes <- getLibsizes(data = ratThymus)

# Adjust the data to remove zeros and rescale by the library scaling
# factors. Convert to log scale.
ratThymus[ratThymus == 0] <- 1
normRT <- log2(t(t(ratThymus / libsizes)) * mean(libsizes))

# run kCluster on reduced set.
normRT <- normRT[1:1000,]
kClust <- kCluster(normRT)

# make the clusters from these data.
mkClust <- makeClusters(kClust, normRT, threshold = 1)

# or using likelihood data from a Bayesian analysis of the data

# load in analysed countData object
data(cD.ratThymus, package = "clusterSeq")

# estimate likelihoods of dissimilarity on reduced set
aM <- associatePosteriors(cD.ratThymus[1:1000,])

# make clusters from dissimilarity data
sX <- makeClusters(aM, cD.ratThymus, threshold = 0.5)

# plot first six clusters
par(mfrow = c(2,3))
plotCluster(sX[1:6], cD.ratThymus)

}

% modification on git from copied files
\name{wallace}
\alias{wallace}

\title{
Computes Wallace scores comparing two clustering methods.
}
\description{
  Given two clusterings A and B we can calculate the likelihood that
  two elements are in the same cluster in B given that they are in the
  same cluster in A, and vice versa.
}
\usage{
wallace(v1, v2)
}

\arguments{
  \item{v1}{
    SimpleIntegerList object (output from makeClusters or makeClustersFF).
}
  \item{v2}{
    SimpleIntegerList object (output from makeClusters or makeClustersFF).
}
}
\value{
Vector of length 2 giving conditional likelihoods.
}
\author{
Thomas J. Hardcastle
}

\examples{

# using likelihood data from a Bayesian analysis of the data

# load in analysed countData object
data(cD.ratThymus, package = "clusterSeq")

# estimate likelihoods of dissimilarity on reduced set
aM <- associatePosteriors(cD.ratThymus[1:1000,])

# make clusters from dissimilarity data
sX <- makeClusters(aM, cD.ratThymus[1:1000,], threshold = 0.5)

# or using k-means clustering on raw count data

#Load in the processed data of observed read counts at each gene for each sample. 
data(ratThymus, package = "clusterSeq")

# Library scaling factors are acquired here using the getLibsizes
# function from the baySeq package.
libsizes <- getLibsizes(data = ratThymus)

# Adjust the data to remove zeros and rescale by the library scaling
# factors. Convert to log scale.
ratThymus[ratThymus == 0] <- 1
normRT <- log2(t(t(ratThymus / libsizes)) * mean(libsizes))

# run kCluster on reduced set.
normRT <- normRT[1:1000,]
kClust <- kCluster(normRT, replicates = cD.ratThymus@replicates)

# make the clusters from these data.
mkClust <- makeClusters(kClust, normRT, threshold = 1)


# compare clusterings
wallace(sX, mkClust)

}

\keyword{manip}

% modification on git from copied files
\name{ratThymus}
\docType{data}
\alias{ratThymus}

\title{Data from female rat thymus tissue taken from the Rat BodyMap
  project (Yu et al, 2014).}

\description{This data set is a matrix ('mobData') of raw count data acquired
  for 17230 genes from 16 samples of female rat thymus tissue. The
  tissues are extracted from four different age groups (2, 6, 21 and 104
  week) with four replicates at each age. Gene annotation is given in
  the rownames of the matrix.
}

\value{A matrix}
\usage{data(ratThymus)}
\format{A matrix of RNA-Seq counts in which each of the sixteen columns
  represents a sample, and each row a gene locus.}
\source{Illumina sequencing.}
\seealso{\code{\link{cD.ratThymus}}}

\references{Yu Y. et al. A rat RNA-Seq transcriptomic BodyMap across 11 organs and 4
  developmental stages. Nature Communications (2014)}

\keyword{datasets}
    

% modification on git from copied files
\name{kCluster}
\alias{kCluster}

\title{Constructs co-expression dissimilarities from k-means analyses.}

\description{
  This function aims to find pairwise distances between genes. It
  does this by constructing k-means clusterings of the observed (log)
  expression for each gene, and for each pair of genes, finding the
  maximum value of k for which the centroids of the clusters are
  monotonic between the genes.
}
\usage{
kCluster(cD, maxK = 100, matrixFile = NULL, replicates =
  NULL, algorithm = "Lloyd", B = 1000, sdm = 1)
}
\arguments{
  \item{cD}{
    A \code{\link[baySeq]{countData}} object containing the raw count
  data for each gene, or a matrix containing the logged and normalised
  values for each gene (rows) and sample (columns).
}
  \item{maxK}{
    The maximum value of k for which k-means clustering will be
  performed. Defaults to 100. 
}
  \item{matrixFile}{
    If given, a file to write the complete (gzipped) matrix of pairwise
  distances between genes. Defaults to NULL.
}
  \item{replicates}{If given, a factor or vector that can be cast to a
  factor that defines the replicate structure of the data. See Details.    
}
  \item{algorithm}{
    The algorithm to be used by the kmeans function.
  }
  \item{B}{
    Number of iterations of bootstrapping algorithm used to establish
  clustering validity
  }
  \item{sdm}{
    Thresholding parameter for validity; see Details.
  }
}
\details{
  In comparing two genes, we find the maximum value of k for which
  separate k-means clusterings of the two genes lead to a monotonic
  relationship between the centroids of the clusters. For this value of
  k, the maximum difference between expression levels observed within a
  cluster of either gene is reported as a measure of the dissimilarity
  between the two genes.

  There is a potential issue in that for genes non-differentially
  expressed across all samples (i.e., the appropriate value of k is 1),
  there will nevertheless exist clusterings for k > 1. For some
  arrangements of data, this leads to misattribution of
  non-differentially expressed genes. We identify these cases by
  adapting Tibshirani's gap statistic; bootstrapping uniformly
  distributed data on the same range as the observed data, calculating
  the dissimilarity score as above, and finding those cases for
  which the gap between the bootstrapped mean dissimilarity and the
  observed dissimilarity for k = 1 exceeds that for k = 2 by more than
  some multiple (sdm) of the standard error of the bootstrapped
  dissimilarities of k = 2. These cases are forced to be treated as
  non-differentially expressed by discarding all dissimilarity data for
  k > 1. 
  
  If the replicates vector is given, or if the replicates slot of a
  \code{\link[baySeq]{countData}} given as the 'cD' variable is
  complete, then the k-means clustering will be done on the median of
  the expression values of each replicate group. Dissimilarity calculations
  will still be made on the full data.
}
\value{
A data.frame which for each gene defines its nearest neighbour of higher
  row index and the dissimilarity with that neighbour.
}
\author{
Thomas J. Hardcastle
}

\seealso{
  \code{\link{makeClusters}}
  \code{\link{makeClustersFF}}
  \code{\link{associatePosteriors}}
}
\examples{
#Load in the processed data of observed read counts at each gene for each sample. 
data(ratThymus, package = "clusterSeq")

# Library scaling factors are acquired here using the getLibsizes
# function from the baySeq package.
libsizes <- getLibsizes(data = ratThymus)

# Adjust the data to remove zeros and rescale by the library scaling
# factors. Convert to log scale.
ratThymus[ratThymus == 0] <- 1
normRT <- log2(t(t(ratThymus / libsizes)) * mean(libsizes))

# run kCluster on reduced set.
normRT <- normRT[1:1000,]
kClust <- kCluster(normRT)
head(kClust)

# Alternatively, run on a count data object:
# load in analysed countData (baySeq package) object
library(baySeq)
data(cD.ratThymus, package = "clusterSeq")

# estimate likelihoods of dissimilarity on reduced set
kClust2 <- kCluster(cD.ratThymus[1:1000,])
head(kClust2)

}

\keyword{manip}

% modification on git from copied files
\name{cD.ratThymus}
\docType{data}
\alias{cD.ratThymus}

\title{Data from female rat thymus tissue taken from the Rat BodyMap
  project (Yu et al, 2014) and processed by baySeq.} 

\description{This data set is a \code{\link[baySeq]{countData}} object
  for 17230 genes from 16 samples of female rat thymus tissue. The
  tissues are extracted from four different age groups (2, 6, 21 and 104
  week) with four replicates at each age. Posterior likelihoods for the
  15 possible patterns of differential expression have been
  precalculated using the \code{\link[baySeq]{baySeq-package}} functions.
}

\value{A \code{\link[baySeq]{countData}} object}
\usage{data(cD.ratThymus)}
\format{A \code{\link[baySeq]{countData}} object}
\source{Illumina sequencing.}
\seealso{\code{\link{ratThymus}}}

\references{Yu Y. et al. A rat RNA-Seq transcriptomic BodyMap across 11
  organs and 4 developmental stages. Nature Communications (2014)}

\keyword{datasets}
    

% modification on git from copied files
\name{associatePosteriors}
\alias{associatePosteriors}
\title{Associates posterior likelihood to generate co-expression
  dissimilarities between genes}

\description{
  This function aims to find pairwise dissimilarities between genes. It
  does this by comparing the posterior likelihoods of patterns of
  differential expression for each gene, and estimating the likelihood
  that the two genes are not equivalently expressed.
}
\usage{
associatePosteriors(cD, maxsize = 250000, matrixFile = NULL)
}

\arguments{
  \item{cD}{
    A \code{\link[baySeq]{countData}} object containing posterior
  likelihoods of differential expression for each gene.
}
  \item{maxsize}{
    The maximum size (in MB) to use when partitioning the data.
}
  \item{matrixFile}{
    If given, a file to write the complete (gzipped) matrix of pairwise
  distances between genes. Defaults to NULL.
}
}

\details{
  In comparing two genes, we find all patterns of expression considered
  in the '@groups' slot of the 'cD' (\code{\link[baySeq]{countData}})
  object for which the expression of the two genes can be considered
  monotonic. We then subtract the sum the posterior likelihods of these
  patterns of expression from 1 to define a likelihood of dissimilarity
  between the two genes.
}
\value{
A data.frame which for each gene defines its nearest neighbour of higher
  row index and the dissimilarity with that neighbour.
}
\author{
Thomas J. Hardcastle
}

\seealso{
  \code{\link{makeClusters}}
  \code{\link{makeClustersFF}}
  \code{\link{kCluster}}
}
\examples{
# load in analysed countData (baySeq package) object
library(baySeq)
data(cD.ratThymus, package = "clusterSeq")

# estimate likelihoods of dissimilarity on reduced set
aM <- associatePosteriors(cD.ratThymus[1:1000,])
}

\keyword{manip}

% modification on git from copied files
\name{plotCluster}
\alias{plotCluster}

\title{Plots data from clusterings.}

\description{
Given clusterings and expression data, plots representative expression
data for each clustering.
}
\usage{
plotCluster(cluster, cD, sampleSize = 1000)
}

\arguments{
  \item{cluster}{
    A list object defining the clusters, produced by \code{\link{makeClusters}} or \code{\link{makeClustersFF}}.
}
  \item{cD}{
    The data object used to produce the clusters.
}
  \item{sampleSize}{
    The maximum number of genes that will be ploted.
}
}
\details{
Expression data are normalised and rescaled before plotting.
}
\value{
Plotting function.
}

\author{
Thomas J Hardcastle
}

\seealso{
  \code{\link{makeClusters}}
  \code{\link{makeClustersFF}}
}
\examples{
# load in analysed countData object
data(cD.ratThymus, package = "clusterSeq")

# estimate likelihoods of dissimilarity on reduced set
aM <- associatePosteriors(cD.ratThymus[1:1000,])

# make clusters from dissimilarity data
sX <- makeClusters(aM, cD.ratThymus, threshold = 0.5)

# plot first six clusters
par(mfrow = c(2,3))
plotCluster(sX[1:6], cD.ratThymus)

}

\keyword{plot}

% modification on git from copied files
\name{makeClustersFF}
\alias{makeClustersFF}

\title{
  Creates clusters from a file containing a full dissimilarity matrix.
}
\description{
  This function uses the complete pairwise dissimilarity scores to
  construct a hierarchical clustering of the genes.
}
\usage{
makeClustersFF(file, method = "complete", cut.height = 5)
}

\arguments{
  \item{file}{
    Filename containing the dissimilarity data.
}
  \item{method}{
    Method to use in \code{\link{hclust}}.
}
  \item{cut.height}{
    Cut height to use in \code{\link{hclust}}.
}
}
\value{
An IntegerList object containing the clusters derived from a cut hierarchical clustering.
}
\author{
Thomas J Hardcastle
}
\seealso{
  \code{\link{makeClusters}}
  \code{\link{kCluster}}
  \code{\link{associatePosteriors}}
}
\examples{
#Load in the processed data of observed read counts at each gene for each sample. 
data(ratThymus, package = "clusterSeq")

# Library scaling factors are acquired here using the getLibsizes
# function from the baySeq package.
libsizes <- getLibsizes(data = ratThymus)

# Adjust the data to remove zeros and rescale by the library scaling
# factors. Convert to log scale.
ratThymus[ratThymus == 0] <- 1
normRT <- log2(t(t(ratThymus / libsizes)) * mean(libsizes))

# run kCluster on reduced set. For speed, one thousand bootstraps are
# used, but higher values should be used in real analyses.
# Write full dissimilarity matrix to file "kclust.gz"
normRT <- normRT[1:1000,]
kClust <- kCluster(normRT, B = 1000, matrixFile = "kclust.gz")


# make the clusters from these data.
mkClustR <- makeClustersFF("kclust.gz")

# no need to clean up (specific to Bioconductor pipeline)
file.remove("kclust.gz")
}


\keyword{manip}
